package com.basic.eyflutter_core.device;

import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.content.ContentResolver;
import android.content.Context;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Environment;
import android.os.StatFs;
import android.provider.Settings;
import android.text.TextUtils;
import android.text.format.Formatter;

import com.basic.eyflutter_core.beans.DeviceResponse;
import com.basic.eyflutter_core.beans.TelephoneInfo;
import com.cloud.eyutils.encryption.Md5Utils;
import com.cloud.eyutils.logs.Logger;
import com.cloud.eyutils.manager.ObjectManager;
import com.cloud.eyutils.utils.ConvertUtils;
import com.cloud.eyutils.utils.GlobalUtils;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Locale;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2020/10/27
 * Description:
 * Modifier:
 * ModifyContent:
 */
@SuppressWarnings("deprecation")
public class DeviceHelper {

    private String[] units = {"B", "KB", "MB", "GB", "TB"};

    public String getDeviceId(Context context, TelephoneInfo telInfo, String androidId) {
        if (TextUtils.isEmpty(androidId)) {
            return androidId;
        }
        if (Build.VERSION.SDK_INT > 28) {
            return genUnique(context);
        }
        if (!TextUtils.isEmpty(telInfo.getMeid())) {
            return telInfo.getMeid();
        }
        if (!TextUtils.isEmpty(telInfo.getImei())) {
            return telInfo.getImei();
        }
        if (!TextUtils.isEmpty(telInfo.getImsi())) {
            return telInfo.getImsi();
        }
        if (!TextUtils.isEmpty(telInfo.getSerial())) {
            return telInfo.getSerial();
        }
        return genUnique(context);
    }

    private String genUnique(Context context) {
        StringBuilder simulationSerialBuilder = new StringBuilder("35");
        simulationSerialBuilder.append(Build.BOARD.length() % 10)
                .append(Build.BRAND.length() % 10)
                .append(Build.CPU_ABI.length() % 10)
                .append(Build.DEVICE.length() % 10)
                .append(Build.DISPLAY.length() % 10)
                .append(Build.HOST.length() % 10)
                .append(Build.ID.length() % 10)
                .append(Build.MANUFACTURER.length() % 10)
                .append(Build.MODEL.length() % 10)
                .append(Build.PRODUCT.length() % 10)
                .append(Build.TAGS.length() % 10)
                .append(Build.TYPE.length() % 10)
                .append(Build.USER.length() % 10);
        if (simulationSerialBuilder.length() == 15) {
            return Md5Utils.encryptForHex(simulationSerialBuilder.toString());
        }
        ContentResolver contentResolver = context.getContentResolver();
        String utdid = Settings.System.getString(contentResolver, "mqBRboGZkQPcAkyk");
        if (!TextUtils.isEmpty(utdid)) {
            return Md5Utils.encryptForHex(utdid);
        }
        String utdidBase64 = Settings.System.getString(contentResolver, "dxCRMxhQkdGePGnp");
        if (!TextUtils.isEmpty(utdidBase64)) {
            return Md5Utils.encryptForHex(utdidBase64);
        }
        return String.format("0x1%s", Md5Utils.encryptForHex(GlobalUtils.getNewGuid()));
    }

    private String getDeviceName() {
        String manufacturer = Build.MANUFACTURER;
        String model = Build.MODEL;
        if (model.startsWith(manufacturer)) {
            return capitalize(model);
        }
        return capitalize(manufacturer) + " " + model;
    }

    private String capitalize(String str) {
        if (TextUtils.isEmpty(str)) {
            return str;
        }
        char[] arr = str.toCharArray();
        boolean capitalizeNext = true;
        String phrase = "";
        for (char c : arr) {
            if (capitalizeNext && Character.isLetter(c)) {
                phrase += Character.toUpperCase(c);
                capitalizeNext = false;
                continue;
            } else if (Character.isWhitespace(c)) {
                capitalizeNext = true;
            }
            phrase += c;
        }
        return phrase;
    }

    private String getLinuxCore_Ver() {
        Process process = null;
        String kernelVersion = "";
        try {
            process = Runtime.getRuntime().exec("cat /proc/version");
        } catch (IOException e) {
            e.printStackTrace();
        }
        InputStream outs = process.getInputStream();
        InputStreamReader isrout = new InputStreamReader(outs);
        BufferedReader brout = new BufferedReader(isrout, 8 * 1024);
        String result = "";
        String line;
        try {
            while ((line = brout.readLine()) != null) {
                result += line;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            if (result != "") {
                String Keyword = "version ";
                int index = result.indexOf(Keyword);
                line = result.substring(index + Keyword.length());
                index = line.indexOf(" ");
                kernelVersion = line.substring(0, index);
            }
        } catch (IndexOutOfBoundsException e) {
            e.printStackTrace();
        }
        return kernelVersion;
    }

    private String getInner_Ver() {
        String ver = "";
        if (Build.DISPLAY.contains(Build.VERSION.INCREMENTAL)) {
            ver = Build.DISPLAY;
        } else {
            ver = Build.VERSION.INCREMENTAL;
        }
        return ver;
    }

    //https://blog.csdn.net/xx326664162/article/details/52438706/
    public DeviceResponse getSysDeviceInfo(Context context, String deviceId, String android) {
        DeviceResponse response = new DeviceResponse();
        response.setDeviceId(deviceId);
        response.setDeviceName(getDeviceName());
        response.setBrand(Build.BRAND);
        response.setSysVersion(Build.VERSION.RELEASE);
        response.setKernelVersion(System.getProperty("os.version"));
        response.setModel(Build.MODEL);
        response.setHardware(Build.HARDWARE);
        response.setCpuCoreNum(String.valueOf(Runtime.getRuntime().availableProcessors()));
        response.setAndroidId(android);

        int screenWidth = ObjectManager.getScreenWidth(context);
        int screenHeight = ObjectManager.getScreenHeight(context);
        response.setResolution(String.format("%sx%s", screenWidth, screenHeight));
        response.setDensity(String.valueOf(context.getResources().getDisplayMetrics().density));
        response.setStorageSize(getStorageSize());
        response.setRunMemory(getTotalMemory(context));
//        response.setRunMemory(String.format("总内存大小=%s；可用内存大小=%s；", getTotalMemory(context), getAvailMemory(context)));
        response.setManufacturer(Build.MANUFACTURER);
        response.setIp(getLocalIpAddress(context));
        response.setHost(Build.HOST);
        response.setAbis(getABIS());
        response.setBoard(Build.BOARD);
        response.setAvailableMemory(getAvailMemory(context));
        response.setAvailableStorageSize(getAvailableStorageSize());
        response.setRemainStorageSize(setRemainStorageSize());
        return response;
    }

    private String getLocalIpAddress(Context context) {
        try {
            WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
            WifiInfo wifiInfo = wifiManager.getConnectionInfo();
            int ipInt = wifiInfo.getIpAddress();
            StringBuilder sb = new StringBuilder();
            sb.append(ipInt & 0xFF).append(".");
            sb.append((ipInt >> 8) & 0xFF).append(".");
            sb.append((ipInt >> 16) & 0xFF).append(".");
            sb.append((ipInt >> 24) & 0xFF);
            return sb.toString();
        } catch (Exception e) {
            Logger.error(e);
        }
        return "";
    }

    private String getTotalMemory(Context context) {
        String str1 = "/proc/meminfo";
        String str2;
        String[] arrayOfString;
        long initial_memory = 0;
        try {
            FileReader localFileReader = new FileReader(str1);
            BufferedReader localBufferedReader = new BufferedReader(localFileReader, 8192);
            //读取meminfo第一行，系统总内存大小
            str2 = localBufferedReader.readLine();
            arrayOfString = str2.split("\\s+");
            // 获得系统总内存，单位是KB
            int i = Integer.valueOf(arrayOfString[1]).intValue();
            //int值乘以1024转换为long类型
            initial_memory = new Long((long) i * 1024);
            localBufferedReader.close();
        } catch (IOException e) {
            Logger.error(e);
        }
        //Byte转换为KB或者MB，内存大小规格化
        return Formatter.formatFileSize(context, initial_memory);
    }

    private String getAvailMemory(Context context) {
        ActivityManager am = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        ActivityManager.MemoryInfo mi = new ActivityManager.MemoryInfo();
        am.getMemoryInfo(mi);
        return Formatter.formatFileSize(context, mi.availMem);
    }

    private String getAvailableStorageSize() {
        StatFs statFs = new StatFs(Environment.getExternalStorageDirectory().getPath());
        long blockSize = statFs.getBlockSize();
        long availableCount = statFs.getAvailableBlocks();
        float availableSize = blockSize * availableCount;
        String available = getUnit(availableSize);
        return available;
    }

    private String setRemainStorageSize() {
        StatFs statFs = new StatFs(Environment.getExternalStorageDirectory().getPath());
        long blockSize = statFs.getBlockSize();
        long availableCount = statFs.getAvailableBlocks();

        long freeBlocks = statFs.getFreeBlocks();
        float availableSize = blockSize * availableCount;
        float freeSize = blockSize * freeBlocks;
        float unavailableSize = freeSize - availableSize;
        String unavailable = getUnit(unavailableSize);
        return unavailable;
    }


    private String getStorageSize() {
        StatFs statFs = new StatFs(Environment.getExternalStorageDirectory().getPath());
        //存储块总数量
        long blockCount = statFs.getBlockCount();
        //块大小
        long blockSize = statFs.getBlockSize();
        long availableCount = statFs.getAvailableBlocks();
        String total = getUnit(blockSize * blockCount);
        return total;
//        return String.format("剩余空间总大小=%s；可用空间大小=%s；保留块(不可用块)大小=%s;", total, available, unavailable);
    }

    private String getUnit(float size) {
        int index = 0;
        while (size > 1024 && index < 4) {
            size = size / 1024;
            index++;
        }
        return String.format(Locale.getDefault(), " %.2f %s", size, units[index]);
    }

    private String getABIS() {
        String[] supported32BitAbis = new String[0];
        if (Build.VERSION.SDK_INT >= 21) {
            supported32BitAbis = Build.SUPPORTED_32_BIT_ABIS;
        }
        String[] supported64BitAbis = new String[0];
        if (Build.VERSION.SDK_INT >= 21) {
            supported64BitAbis = Build.SUPPORTED_64_BIT_ABIS;
        }
        String[] array = new String[supported32BitAbis.length + supported64BitAbis.length];
        int position = 0;
        for (String supported32BitAbi : supported32BitAbis) {
            array[position] = supported32BitAbi;
            position++;
        }
        for (String supported64BitAbi : supported64BitAbis) {
            array[position] = supported64BitAbi;
            position++;
        }
        return ConvertUtils.toJoin(array, ",");
    }

    @SuppressLint("HardwareIds")
    public String getAndroidId(Context context) {
        try {
            return Settings.Secure.getString(context.getContentResolver(), Settings.Secure.ANDROID_ID);
        } catch (Exception e) {
            return "";
        }
    }
}
