package com.basic.eyflutter_core.device;

import android.annotation.SuppressLint;
import android.content.Context;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.text.TextUtils;

import com.cloud.eyutils.utils.ObjectJudge;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.Reader;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2020/10/2
 * Description:
 * Modifier:
 * ModifyContent:
 */
@SuppressWarnings("deprecation")
public class MacHelper {

    public String getMacAddress(Context context) {
        String mac = "";
        try {
            if (Build.VERSION.SDK_INT < 23) {
                //6.0以下
                mac = getMacAddress0(context);
            } else {
                String address = get6t7MacAddress(context);
                if (!TextUtils.isEmpty(address)) {
                    mac = address;
                }
                String hardwareAddress = getMachineHardwareAddress();
                if (!TextUtils.isEmpty(hardwareAddress)) {
                    return hardwareAddress;
                }
                mac = getLocalMacAddressFromBusybox();
            }
            mac = (mac == null ? "" : mac.trim().toLowerCase());
            return mac;
        } catch (Exception e) {
            return "";
        }
    }

    @SuppressLint("HardwareIds")
    private static String getMacAddress0(Context context) {
        try {
            WifiManager wifi = (WifiManager) context.getApplicationContext().getSystemService(Context.WIFI_SERVICE);
            boolean flag = false;
            if (!wifi.isWifiEnabled()) {
                flag = true;
            }
            if (flag) {
                wifi.setWifiEnabled(true);
            }
            WifiInfo connectionInfo = wifi.getConnectionInfo();
            if (connectionInfo == null) {
                return "";
            }
            @SuppressLint("HardwareIds")
            String macAddress = connectionInfo.getMacAddress();
            if (flag) {
                wifi.setWifiEnabled(false);
            }
            return macAddress;
        } catch (Exception e) {
            return "";
        }
    }

    private static String get6t7MacAddress(Context context) {
        // 如果是6.0以下，直接通过wifimanager获取
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            String macAddress0 = getMacAddress0(context);
            if (!TextUtils.isEmpty(macAddress0)) {
                return macAddress0;
            }
        }
        String str = "";
        String macSerial = "";
        try {
            Process pp = Runtime.getRuntime().exec(
                    "cat /sys/class/net/wlan0/address");
            InputStreamReader ir = new InputStreamReader(pp.getInputStream());
            LineNumberReader input = new LineNumberReader(ir);
            for (; null != str; ) {
                str = input.readLine();
                if (str != null) {
                    macSerial = str.trim();// 去空格
                    break;
                }
            }
        } catch (Exception ex) {
            //exception
        }
        if (macSerial == null || "".equals(macSerial)) {
            try {
                return loadFileAsString("/sys/class/net/eth0/address")
                        .toUpperCase().substring(0, 17);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return macSerial;
    }

    private static String loadFileAsString(String fileName) throws Exception {
        FileReader reader = new FileReader(fileName);
        String text = loadReaderAsString(reader);
        reader.close();
        return text;
    }

    private static String loadReaderAsString(Reader reader) throws Exception {
        StringBuilder builder = new StringBuilder();
        char[] buffer = new char[4096];
        int readLength = reader.read(buffer);
        while (readLength >= 0) {
            builder.append(buffer, 0, readLength);
            readLength = reader.read(buffer);
        }
        return builder.toString();
    }

    private static String getMachineHardwareAddress() {
        try {
            Enumeration<NetworkInterface> interfaces = null;
            try {
                interfaces = NetworkInterface.getNetworkInterfaces();
            } catch (SocketException e) {
                return "";
            }
            if (interfaces == null) {
                return "";
            }
            String hardWareAddress = "";
            NetworkInterface networkInterface = null;
            while (interfaces.hasMoreElements()) {
                networkInterface = interfaces.nextElement();
                if (networkInterface == null) {
                    continue;
                }
                try {
                    hardWareAddress = bytesToString(networkInterface.getHardwareAddress());
                    if (!TextUtils.isEmpty(hardWareAddress)) {
                        break;
                    }
                } catch (SocketException e) {
                    e.printStackTrace();
                }
            }
            if (networkInterface != null &&
                    TextUtils.equals("wlan0", networkInterface.getName()) &&
                    !TextUtils.isEmpty(hardWareAddress)) {
                hardWareAddress = hardWareAddress.replaceAll(":", "");
            }
            return hardWareAddress;
        } catch (Exception e) {
            return "";
        }
    }

    private static String bytesToString(byte[] bytes) {
        if (ObjectJudge.isNullOrEmpty(bytes)) {
            return null;
        }
        StringBuilder buf = new StringBuilder();
        for (byte b : bytes) {
            buf.append(String.format("%02X:", b));
        }
        if (buf.length() > 0) {
            buf.deleteCharAt(buf.length() - 1);
        }
        return buf.toString();
    }

    private static String getLocalMacAddressFromBusybox() {
        String result = "";
        String Mac = "";
        result = callCmd("busybox ifconfig", "HWaddr");
        // 如果返回的result == null，则说明网络不可取
        if (result == null) {
            return "";
        }
        // 对该行数据进行解析
        // 例如：eth0 Link encap:Ethernet HWaddr 00:16:E8:3E:DF:67
        if (result.length() > 0 && result.contains("HWaddr") == true) {
            Mac = result.substring(result.indexOf("HWaddr") + 6,
                    result.length() - 1);
            result = Mac;
        }
        return result;
    }

    private static String callCmd(String cmd, String filter) {
        String result = "";
        String line = "";
        try {
            Process proc = Runtime.getRuntime().exec(cmd);
            InputStreamReader is = new InputStreamReader(proc.getInputStream());
            BufferedReader br = new BufferedReader(is);
            while ((line = br.readLine()) != null
                    && line.contains(filter) == false) {
                result += line;
            }
            result = line;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }
}
