package com.basic.eyflutter_core.device;

import android.Manifest;
import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Build;
import android.telephony.TelephonyManager;
import android.text.TextUtils;

import com.basic.eyflutter_core.beans.TelephoneInfo;
import com.cloud.eyutils.logs.Logger;
import com.cloud.eyutils.utils.ValidUtils;
import com.yanzhenjie.permission.AndPermission;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2020/10/2
 * Description:
 * Modifier:
 * ModifyContent:
 */
@SuppressWarnings("deprecation")
public class TelephoneHelper {

    public TelephoneInfo getTelInfo(Context context) {
        TelephoneInfo telephoneInfo = new TelephoneInfo();
        TelephonyManager telephonyManager = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
        telephoneInfo.setImei(getImei(context, telephonyManager));
        telephoneInfo.setImsi(getImsi(context, telephonyManager));
        telephoneInfo.setMeid(getMeid(context, telephonyManager));
        telephoneInfo.setSerial(getSerialNumber(context, telephonyManager));
        if (TextUtils.isEmpty(telephoneInfo.getImei()) &&
                TextUtils.isEmpty(telephoneInfo.getImsi()) &&
                TextUtils.isEmpty(telephoneInfo.getMeid()) &&
                TextUtils.isEmpty(telephoneInfo.getSerial())) {
            getDeviceStates(telephoneInfo);
        }
        return telephoneInfo;
    }

    private void getDeviceStates(TelephoneInfo telephoneInfo) {
        try {
            if (Build.VERSION.SDK_INT > 28) {
                return;
            }
            // get properties
            Process pp = Runtime.getRuntime().exec("getprop");
            InputStreamReader ir = new InputStreamReader(pp.getInputStream());
            LineNumberReader reader = new LineNumberReader(ir);
            for (int i = 1; i < Integer.MAX_VALUE; i++) {
                String line;
                try {
                    line = reader.readLine();
                } catch (IOException e) {
                    continue;
                }
                if (line == null) {
                    break;
                }
                if (TextUtils.isEmpty(line)) {
                    continue;
                }
                line = line.trim().toLowerCase();
                if (TextUtils.isEmpty(telephoneInfo.getMeid()) && line.contains("meid")) {
                    String value = getProcessValue(line);
                    telephoneInfo.setMeid(value);
                } else if (TextUtils.isEmpty(telephoneInfo.getSerial()) && (line.contains("gsm.serial") || line.contains("ro.serialno") || line.contains("ro.boot.serialno"))) {
                    String value = getProcessValue(line);
                    String[] lst = value.split(" ");
                    String serial = lst.length > 0 ? lst[0].trim() : "";
                    telephoneInfo.setSerial(serial);
                } else if (TextUtils.isEmpty(telephoneInfo.getImei()) && line.contains("imei")) {
                    String value = getProcessValue(line);
                    if (value.length() > 6) {
                        if (value.matches("[0-9]+")) {
                            telephoneInfo.setImei(value);
                        }
                    }
                } else if (TextUtils.isEmpty(telephoneInfo.getImsi()) && line.contains("imsi")) {
                    String value = getProcessValue(line);
                    if (value.length() > 6) {
                        telephoneInfo.setImsi(value);
                    }
                }
                if (!TextUtils.isEmpty(telephoneInfo.getMeid()) ||
                        !TextUtils.isEmpty(telephoneInfo.getImei()) ||
                        !TextUtils.isEmpty(telephoneInfo.getImsi()) ||
                        !TextUtils.isEmpty(telephoneInfo.getSerial())) {
                    break;
                }
            }
        } catch (Exception e) {
            Logger.error(e);
        }
    }

    private static String getProcessValue(String line) {
        String[] split = line.split(":");
        String value = "";
        if (split.length > 1) {
            value = split[1].trim();
            value = ValidUtils.matche("[^\\[](.)*[^\\]]", value);
        }
        //部分多值的情况以,号分隔
        if (value.length() > 0) {
            String[] vs = value.split(",");
            for (String v : vs) {
                if (!TextUtils.isEmpty(v)) {
                    value = v;
                    break;
                }
            }
        }
        return value;
    }

    @SuppressLint("HardwareIds")
    private String getSerialNumber(Context context, TelephonyManager telephonyManager) {
        try {
            String serialNumber = "";
            if (AndPermission.hasPermissions(context, Manifest.permission.READ_PHONE_STATE)) {
                serialNumber = telephonyManager.getSimSerialNumber();
            }
            return serialNumber;
        } catch (Exception e) {
            try {
                if (Build.VERSION.SDK_INT >= 26) {
                    if (AndPermission.hasPermissions(context, Manifest.permission.READ_PHONE_STATE)) {
                        return Build.getSerial();
                    }
                    return "";
                }
            } catch (Exception e1) {
                //READ_PHONE_STATE
            }
            String serial = Build.SERIAL;
            return TextUtils.equals(serial, "unknown") ? "" : serial;
        }
    }

    private String getMeid(Context context, TelephonyManager telephonyManager) {
        String meid = "";
        try {
            if (Build.VERSION.SDK_INT >= 26) {
                if (AndPermission.hasPermissions(context, Manifest.permission.READ_PHONE_STATE)) {
                    meid = telephonyManager.getMeid();
                }
            }
        } catch (Exception e) {
            return "";
        }
        return meid;
    }

    @SuppressLint("HardwareIds")
    private String getImsi(Context context, TelephonyManager telephonyManager) {
        String subscriberId = "";
        try {
            if (AndPermission.hasPermissions(context, Manifest.permission.READ_PHONE_STATE)) {
                subscriberId = telephonyManager.getSubscriberId();
            }
        } catch (Exception e) {
            return "";
        }
        return subscriberId;
    }

    @SuppressLint("HardwareIds")
    private String getImei(Context context, TelephonyManager telephonyManager) {
        try {
            String imei = "";
            if (Build.VERSION.SDK_INT >= 26) {
                if (AndPermission.hasPermissions(context, Manifest.permission.READ_PHONE_STATE)) {
                    imei = telephonyManager.getImei();
                }
            }
            if (TextUtils.isEmpty(imei)) {
                if (AndPermission.hasPermissions(context, Manifest.permission.READ_PHONE_STATE)) {
                    imei = telephonyManager.getDeviceId();
                }
            }
            return imei;
        } catch (Exception e) {
            return "";
        }
    }
}
