package com.basic.eyflutter_core.subscribe;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;

import com.basic.eyflutter_core.channel.OnDistributionSubscribe;
import com.cloud.eyutils.launchs.LauncherState;
import com.cloud.eyutils.utils.ConvertUtils;
import com.cloud.eyutils.utils.GlobalUtils;
import com.cloud.eyutils.utils.JsonUtils;

import java.util.HashMap;

import io.flutter.plugin.common.MethodChannel;

public class ApplicationInfoSubscribe extends OnDistributionSubscribe {

    @Override
    public void onSubscribe(MethodChannel.Result result, HashMap<String, Object> arguments) {
        HashMap<String, Object> map = new HashMap<>();
        Context applicationContext = LauncherState.getApplicationContext();
        String packageName = applicationContext.getPackageName();
        map.put("channel", getChannelName(applicationContext, packageName));
        String buildConfigPath = String.format("%s.BuildConfig", packageName);
        Object versionCode = GlobalUtils.getBuildConfigValue(buildConfigPath, "VERSION_CODE");
        map.put("build", ConvertUtils.toInt(versionCode));
        Object versionName = GlobalUtils.getBuildConfigValue(buildConfigPath, "VERSION_NAME");
        map.put("version", ConvertUtils.toString(versionName));
        map.put("packageName", packageName);
        String json = JsonUtils.toJson(map);
        result.success(json);
    }

    public static String getChannelName(Context applicationContext, String packageName) {
        String channelName = "";
        try {
            PackageManager packageManager = applicationContext.getPackageManager();
            if (packageManager != null) {
                ApplicationInfo applicationInfo = packageManager.getApplicationInfo(packageName, PackageManager.GET_META_DATA);
                if (applicationInfo != null) {
                    if (applicationInfo.metaData != null) {
                        channelName = applicationInfo.metaData.getString("EY_CHANNEL");
                    }
                }
            }
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return channelName;
    }
}
