package com.basic.eyflutter_core.subscribe;

import android.text.TextUtils;

import com.basic.eyflutter_core.ChannelConstants;
import com.basic.eyflutter_core.channel.ChannelPlugin;
import com.basic.eyflutter_core.channel.OnDistributionSubscribe;
import com.basic.eyflutter_core.nets.OkRxManager;
import com.basic.eyflutter_core.nets.beans.CompleteResponse;
import com.basic.eyflutter_core.nets.beans.FlutterDataResponse;
import com.basic.eyflutter_core.nets.beans.ResponseData;
import com.basic.eyflutter_core.nets.beans.RetrofitParams;
import com.basic.eyflutter_core.nets.beans.SuccessResponse;
import com.basic.eyflutter_core.nets.beans.TransParams;
import com.basic.eyflutter_core.nets.enums.RequestContentType;
import com.basic.eyflutter_core.nets.enums.RequestState;
import com.basic.eyflutter_core.nets.enums.RequestType;
import com.basic.eyflutter_core.nets.enums.ResponseDataType;
import com.basic.eyflutter_core.nets.properties.ByteRequestItem;
import com.cloud.eyutils.HandlerManager;
import com.cloud.eyutils.events.Action1;
import com.cloud.eyutils.events.RunnableParamsN;
import com.cloud.eyutils.storage.files.FilenameUtils;
import com.cloud.eyutils.storage.files.StorageUtils;
import com.cloud.eyutils.utils.ConvertUtils;
import com.cloud.eyutils.utils.JsonUtils;
import com.cloud.eyutils.utils.ObjectJudge;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import io.flutter.plugin.common.MethodChannel;

@SuppressWarnings("unchecked")
public class NetRequestSubscribe extends OnDistributionSubscribe {

    private Map<String, MethodChannel.Result> resultMap = new HashMap<>();

    @Override
    public void onSubscribe(MethodChannel.Result result, HashMap<String, Object> arguments) {
        String method = ConvertUtils.toString(arguments.get("method"));
        String requestId = ConvertUtils.toString(arguments.get("requestId"));
        String contentType = ConvertUtils.toString(arguments.get("contentType"));
        String url = ConvertUtils.toString(arguments.get("url"));
        HashMap<String, Object> data = (HashMap<String, Object>) arguments.get("data");
        HashMap<String, String> headers = (HashMap<String, String>) arguments.get("headers");
        resultMap.put(requestId, result);
        if (TextUtils.equals(method, "GET")) {
            request(requestId, url, contentType, RequestType.GET, headers, data, false);
        } else if (TextUtils.equals(method, "PUT")) {
            request(requestId, url, contentType, RequestType.PUT, headers, data, false);
        } else if (TextUtils.equals(method, "POST")) {
            request(requestId, url, contentType, RequestType.POST, headers, data, false);
        } else if (TextUtils.equals(method, "DELETE")) {
            request(requestId, url, contentType, RequestType.DELETE, headers, data, false);
        } else if (TextUtils.equals(method, "UPLOAD")) {
            request(requestId, url, contentType, RequestType.BYTES, headers, data, true);
        } else if (TextUtils.equals(method, "DOWNLOAD")) {
            download(requestId, url, data);
        } else {
            responseHanndler(requestId, "error", "");
        }
    }

    private void responseHanndler(String requestId, String type, String data) {
        FlutterDataResponse response = new FlutterDataResponse();
        response.setType(type);
        response.setData(data);
        String content = JsonUtils.toJson(response);
        HandlerManager.getInstance().post(new RunnableParamsN<String>() {
            @Override
            public void run(String... params) {
                MethodChannel.Result result = resultMap.get(params[0]);
                if (result != null) {
                    result.success(params[1]);
                }
                resultMap.remove(params[0]);
            }
        }, requestId, content);
    }

    private void download(String requestId, String url, HashMap<String, Object> data) {
        File rootDir = StorageUtils.getRootDir();
        String fileName = FilenameUtils.getName(url);
        File file = StorageUtils.getFile(rootDir, fileName, true);
        OkRxManager.getInstance().download(url, null, null, file, (Action1<Float>) progress -> {
            notifyProcess(requestId, progress);
        }, (Action1<File>) successFile -> {
            Map<String, String> resultMap = new HashMap<>();
            resultMap.put("path", successFile.getAbsolutePath());
            String resultJson = JsonUtils.toJson(resultMap);
            responseHanndler(requestId, "success", resultJson);
        }, (Action1<RequestState>) completeState -> {
            if (completeState == RequestState.Error) {
                responseHanndler(requestId, "error", "");
            }
        });
    }

    private void notifyProcess(String requestId, Float progress) {
        Map<String, Object> resultArgs = new HashMap<>();
        resultArgs.put("requestId", requestId);
        resultArgs.put("progress", progress);
        HandlerManager.getInstance().post(new RunnableParamsN<Map<String, Object>>() {
            @Override
            public void run(Map<String, Object>... params) {
                ChannelPlugin.getInstance().invokeMethod(ChannelConstants.netRequestProcessMethodName, params[0], null);
            }
        }, resultArgs);
    }

    private void request(String requestId, String url, String contentType, RequestType requestType, HashMap<String, String> headers, HashMap<String, Object> data, boolean isUpload) {
        TransParams transParams = new TransParams();
        transParams.setUrl(url);
        RetrofitParams retrofitParams = new RetrofitParams();
        if (TextUtils.equals(contentType, "form")) {
            retrofitParams.setRequestContentType(RequestContentType.Form);
        } else {
            retrofitParams.setRequestContentType(RequestContentType.Json);
        }
        retrofitParams.setRequestType(requestType);
        retrofitParams.setRequestUrl(url);
        retrofitParams.setResponseDataType(ResponseDataType.object);
        if (!ObjectJudge.isNullOrEmpty(headers)) {
            HashMap<String, String> headParams = retrofitParams.getHeadParams();
            headParams.putAll(headers);
            transParams.setHeaders(headers);
        }
        if (!ObjectJudge.isNullOrEmpty(data)) {
            TreeMap<String, Object> params = retrofitParams.getParams();
            params.putAll(data);
        }
        transParams.setRetrofitParams(retrofitParams);
        if (isUpload) {
            ByteRequestItem requestItem = new ByteRequestItem();
            TreeMap<String, Object> params = retrofitParams.getParams();
            String fileType = ConvertUtils.toString(params.get("contentType"));
            if (!TextUtils.isEmpty(fileType)) {
                requestItem.setMediaTypeValue(fileType);
            }
            requestItem.setFieldName("file");
            String filePath = ConvertUtils.toString(params.get("file"));
            File file = new File(filePath);
            if (!file.exists()) {
                responseHanndler(requestId, "error", "");
            } else {
                byte[] bytes = file2byte(file);
                if (bytes == null) {
                    responseHanndler(requestId, "error", "");
                } else {
                    requestItem.setBs(bytes);
                }
            }
            OkRxManager.getInstance().uploadByte(url, headers, data, requestItem, retrofitParams, new SuccessAction(requestId), new CompleteAction(requestId));
        } else {
            OkRxManager.getInstance().request(transParams, new SuccessAction(requestId), new CompleteAction(requestId));
        }
    }

    private byte[] file2byte(File tradeFile) {
        byte[] buffer = null;
        try {
            FileInputStream fis = new FileInputStream(tradeFile);
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            byte[] b = new byte[1024];
            int n;
            while ((n = fis.read(b)) != -1) {
                bos.write(b, 0, n);
            }
            fis.close();
            bos.close();
            buffer = bos.toByteArray();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return buffer;
    }

    private class SuccessAction implements Action1<SuccessResponse> {

        private String requestId;

        public SuccessAction(String requestId) {
            this.requestId = requestId;
        }

        @Override
        public void call(SuccessResponse response) {
            ResponseData responseData = response.getResponseData();
            responseHanndler(requestId, "success", responseData.getResponse());
        }
    }

    private class CompleteAction implements Action1<CompleteResponse> {

        private String requestId;

        public CompleteAction(String requestId) {
            this.requestId = requestId;
        }

        @Override
        public void call(CompleteResponse response) {
            if (response.getRequestState() == RequestState.Error) {
                responseHanndler(requestId, "error", "");
            }
        }
    }
}
