import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_core_example/test_api.dart';
import 'package:flutter/material.dart';

class NetPage extends StatefulWidget {
  const NetPage({Key? key}) : super(key: key);

  @override
  _NetPageState createState() => _NetPageState();
}

class RequestConfig with OnRequestInterceptor {}

class _NetPageState extends State<NetPage> {
  @override
  void initState() {
    DioManager.instance.addInterceptor(type: InterceptorType.CONFIG, interceptor: RequestConfig());
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        backgroundColor: Colors.white,
        appBar: AppBar(
          title: const Text('网络请求'),
        ),
        body: ConstListView(
          itemCount: 20,
          itemExtent: 50,
          buttons: [
            ConstButton(
              text: "GET请求",
              onLongPress: getRequest,
            ),
            ConstButton(
              text: "PUT请求",
              onLongPress: putRequest,
            ),
            ConstButton(
              text: "POST请求",
              onLongPress: postRequest,
            ),
            ConstButton(
              text: "DELETE请求",
              onLongPress: deleteRequest,
            ),
            ConstButton(
              text: "获取配置表数据",
              onLongPress: getConfigTableData,
            ),
            ConstButton(
              text: "文件下载",
              onLongPress: download,
            )
          ],
        ),
      ),
      top: false,
    );
  }

  void getConfigTableData() {
    ConfigTable.instance.getValue(configKey: "texts_version").then((value) {
      Logger.instance.info(value);
    });
  }

  void deleteRequest() {
    DioManager.instance.request(
        retrofit: TestApi.deleteRequest,
        successCall: (String requestId, Map<String, dynamic> dataMap) {
          Logger.instance.info(dataMap);
        },
        completeCall: (String requestId) {
          Logger.instance.info("request complete");
        });
  }

  void postRequest() {
    DioManager.instance.request(
        retrofit: TestApi.postRequest,
        successCall: (String requestId, Map<String, dynamic> dataMap) {
          Logger.instance.info(dataMap);
        },
        completeCall: (String requestId) {
          Logger.instance.info("request complete");
        });
  }

  void putRequest() {
    DioManager.instance.request(
        retrofit: TestApi.putRequest,
        successCall: (String requestId, Map<String, dynamic> dataMap) {
          Logger.instance.info(dataMap);
        },
        completeCall: (String requestId) {
          Logger.instance.info("request complete");
        });
  }

  void getRequest() {
    DioManager.instance.request(
        retrofit: TestApi.getRequest,
        params: {
          "name": "易由信息",
          "domain": "www.geease.com",
        },
        successCall: (String requestId, Map<String, dynamic> dataMap) {
          Logger.instance.info(dataMap);
        },
        completeCall: (String requestId) {
          Logger.instance.info("request complete");
        });
  }

  void download() {
    DioManager.instance.download(
        url: "https://geease-days.oss-cn-hangzhou.aliyuncs.com/pdf/8487cd837217f1e21b93f4e2f5ab8969.pdf",
        processCall: (double process) {
          Logger.instance.info("下载进度：$process");
        },
        successCall: (String path) {
          Logger.instance.info(path);
        },
        completeCall: (requestId) {
          Logger.instance.info("download request complete");
        });
  }
}
