#import "EyflutterCorePlugin.h"
#if __has_include(<eyflutter_core/eyflutter_core-Swift.h>)
#import <eyflutter_core/eyflutter_core-Swift.h>
#else
// Support project import fallback if the generated compatibility header
// is not copied when this plugin is created as a library.
// https://forums.swift.org/t/swift-static-libraries-dont-copy-generated-objective-c-header/19816
#import "eyflutter_core-Swift.h"
#endif

#import <MMKV/MMKV.h>
#import "device/DeviceUtils.h"
#import "device/DeviceBundleInfo.h"

NSString *mmkvUserKey = @"70746218ab58dc96";
NSString *mmkvOrdinaryKey = @"c694f6da00968d67";
NSString *mmkvStateKey = @"fa7463fbf7bc5a65";

static MMKV *userMmkv;
static MMKV *ordinaryMmkv;
static MMKV *stateMmkv;

@implementation EyflutterCorePlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    [SwiftEyflutterCorePlugin registerWithRegistrar:registrar];
    //init mmkv
    [self initMmkv];
}

+ (void)initMmkv {
    NSString *mmkvGroupId = @"com.cloud.mmkv";
    NSString *groupDir = [[NSFileManager defaultManager] containerURLForSecurityApplicationGroupIdentifier:mmkvGroupId].path;
    [MMKV initializeMMKV:nil groupDir:groupDir logLevel:MMKVLogInfo];
    [self checkMmkv];
}

+ (void)checkMmkv {
    if (ordinaryMmkv == nil) {
        ordinaryMmkv = [MMKV defaultMMKV];
    }
    if (userMmkv == nil) {
        userMmkv = [MMKV mmkvWithID:mmkvUserKey];
    }
    if (stateMmkv == nil) {
        stateMmkv = [MMKV mmkvWithID:mmkvStateKey];
    }
}

+ (void)cleanMmkv {
    ordinaryMmkv = nil;
    userMmkv = nil;
    stateMmkv = nil;
}

+ (id)getMMKVInstance:(NSString *)withId {
    if ([withId isEqual:mmkvUserKey]) {
        return userMmkv;
    } else if ([withId isEqual:mmkvStateKey]) {
        return stateMmkv;
    } else {
        return ordinaryMmkv;
    }
}

//缓存数据
+ (void) cacheData:(NSDictionary*) params
onMmkvIntCall:(MmkvIntCall) intCall
onMmkvDoubleCall:(MmkvDoubleCall) doubleCall
onMmkvStringCall:(MmkvStringCall) stringCall
onMmkvBoolCall:(MmkvBoolCall) boolCall; {
    if (params == nil) {
        return;
    }
    NSString *callKey = [params valueForKey:@"flutter_result_call_key"];
    NSString *withId = [params valueForKey:@"withId"];
    NSString *key = [params valueForKey:@"key"];
    NSString *type = [params valueForKey:@"type"];
    NSString *value = [params valueForKey:@"value"];
    if (key == nil || key.length == 0 || value == nil) {
        return;
    }
    MMKV *_mmkv = [self getMMKVInstance:withId];
    @try {
        if ([type isEqual:@"int"]) {
            [_mmkv setInt32:[value intValue] forKey:key];
            
            intCall(callKey,[value intValue]);
            
        } else if ([type isEqual:@"double"]) {
            [_mmkv setDouble:[value doubleValue] forKey:key];
            
            doubleCall(callKey,[value doubleValue]);
            
        } else if ([type isEqual:@"string"]) {
            [_mmkv setString:value forKey:key];
            
            stringCall(callKey,value);
        
        } else if ([type isEqual:@"bool"]) {
            [_mmkv setBool:[value boolValue] forKey:key];
            
            boolCall(callKey,[value boolValue]);
            
        }
    } @catch (NSException *exception) {
        NSLog(@"%@", exception);
    }
}

+ (void)takeData:(NSDictionary *)params onMmkvIntCall:(MmkvIntCall)intCall onMmkvDoubleCall:(MmkvDoubleCall)doubleCall onMmkvStringCall:(MmkvStringCall)stringCall onMmkvBoolCall:(MmkvBoolCall)boolCall {
    if (params == nil) {
        return;
    }
    NSString *callKey = [params valueForKey:@"flutter_result_call_key"];
    NSString *withId = [params valueForKey:@"withId"];
    NSString *key = [params valueForKey:@"key"];
    NSString *type = [params valueForKey:@"type"];
    MMKV *_mmkv = [self getMMKVInstance:withId];
    @try {
        if ([type isEqual:@"int"]) {
            intCall(callKey, [_mmkv getInt32ForKey:key]);
        } else if ([type isEqual:@"double"]) {
            doubleCall(callKey, [_mmkv getDoubleForKey:key]);
        } else if ([type isEqual:@"string"]) {
            stringCall(callKey, [_mmkv getStringForKey:key]);
        } else if ([type isEqual:@"bool"]) {
            boolCall(callKey, [_mmkv getBoolForKey:key]);
        }
    } @catch (NSException *exception) {
        NSLog(@"%@", exception);
    }
}

+ (void)removeMmkvKey:(NSDictionary *)params
     onMmkvRemoveCall:(MmkvRemoveCall) removeCall{
    NSString *callKey = [params valueForKey:@"flutter_result_call_key"];
    NSString *withId = [params valueForKey:@"withId"];
    NSString *key = [params valueForKey:@"key"];
    MMKV *_mmkv = [self getMMKVInstance:withId];
    @try {
        [_mmkv removeValueForKey:key];
        removeCall(callKey,withId);
    } @catch (NSException *exception) {
        NSLog(@"%@", exception);
    }
}

+ (NSString*)getString:(NSString *)key{
    MMKV *_mmkv = [self getMMKVInstance:mmkvOrdinaryKey];
    NSString *result = [_mmkv getStringForKey:key];
    if (result == nil) {
        return @"";
    }
    return result;
}

+ (NSDictionary *)getDeviceResponse {
    NSDictionary *deviceMap = [[NSMutableDictionary alloc] init];
    DeviceBundleInfo* bundle = [DeviceUtils getDeviceOrBundleInfo];
    [deviceMap setValue:[bundle getUniqueGlobalDeviceId] forKey:@"deviceId"];
    [deviceMap setValue:[bundle getDeviceName] forKey:@"deviceName"];
    [deviceMap setValue:[bundle getSysName] forKey:@"systemName"];
    [deviceMap setValue:[bundle getSysVersion] forKey:@"sysVersion"];
    [deviceMap setValue:[bundle getModel] forKey:@"model"];
    [deviceMap setValue:[bundle getManufacturer] forKey:@"manufacturer"];
    [deviceMap setValue:[bundle getIp] forKey:@"ip"];
    [deviceMap setValue:[bundle getBrand] forKey:@"brand"];
    [deviceMap setValue:[bundle getDisplay] forKey:@"resolution"];
    [deviceMap setValue:[bundle getDensity] forKey:@"density"];
    [deviceMap setValue:[bundle getSysLanguage] forKey:@"language"];
    return deviceMap;
}

+ (NSDictionary *)getAppInf:(NSDictionary *)params {
    NSDictionary* infoDict = [[NSBundle mainBundle] infoDictionary];
    //build
    NSString* buildVersion = [infoDict objectForKey:@"CFBundleVersion"];
    //version
    NSString* version = [infoDict objectForKey:@"CFBundleShortVersionString"];
    //package
    NSString* packageName = [infoDict objectForKey:@"CFBundleIdentifier"];
    NSDictionary *dict = [[NSMutableDictionary alloc] init];
    [dict setValue:version forKey:@"version"];
    [dict setValue:buildVersion forKey:@"build"];
    [dict setValue:@"AppStore" forKey:@"channel"];
    [dict setValue:packageName forKey:@"packageName"];
    return dict;
}

@end
