//
//  DioManager.swift
//  eyflutter_core
//
//  Created by 李敬欢 on 2021/12/27.
//

import Foundation
import Alamofire

class DioManager: NSObject {
    
    func request(params: RequestParams, method: HTTPMethod = HTTPMethod.get) -> Void {
        let helper = DioHelper.init();
        let _requestHeaders = helper.toHTTPHeaders(headers: params.headers, contentType: params.contentType);
        let _requestData = helper.toParameters(params: params.data);
        EyNetManager.shareManager().request(params.url,method: method,parameters: _requestData,headers: _requestHeaders).responseJSON { response in
            switch response.result {
            case .success(let responseData):
                let jsonData = JsonUtils.getJSONStringFromDictionary(dataObject: responseData);
                self.responseHanndler(callKey: params.callKey, type: "success", data: jsonData);
                self.notifyComplete(requestId: params.requestId);
                break;
            case .failure(_):
                self.responseHanndler(callKey: params.callKey, type: "error", data: "");
                self.notifyComplete(requestId: params.requestId);
                break;
            }
        };
    }
    
    func upload(params: RequestParams) -> Void {
        EyNetManager.shareManager().upload(multipartFormData: { (formdata) in
            let filePath = params.data["file"] as! String;
            let fileUrl = URL(string: filePath);
            let fileData = try! Data(contentsOf: URL(fileURLWithPath: filePath));
            formdata.append(fileData, withName: "file", fileName: fileUrl!.lastPathComponent, mimeType: params.contentType);
            for (mk,mv) in params.data {
                if mk == "file" {
                    continue;
                }
                if let content = mv as? String {
                    formdata.append(content.data(using: String.Encoding.utf8)!, withName: mk);
                }
            }
        }, to: params.url, headers: params.headers) { encodingResult in
            switch encodingResult {
                case .success(let upload, _, _):
                    upload.uploadProgress { (progress) in
                        let map = ["progress" : progress.fractionCompleted, "requestId" : params.requestId] as [String : Any];
                        ChannelPlugin.invokeMethod(action: "f4f4cce92307e7f5", params: map);
                    }.responseJSON { (response) in
                        if let jsonValue = response.result.value {
                            let jsonData = JsonUtils.getJSONStringFromDictionary(dataObject: jsonValue);
                            self.responseHanndler(callKey: params.callKey, type: "success", data: jsonData);
                            self.notifyComplete(requestId: params.requestId);
                        } else {
                            self.responseHanndler(callKey: params.callKey, type: "error", data: "");
                            self.notifyComplete(requestId: params.requestId);
                        }
                    }
                    break;
                case .failure(_):
                    self.responseHanndler(callKey: params.callKey, type: "error", data: "");
                    self.notifyComplete(requestId: params.requestId);
                    break;
            }
        }
    }
    
    func download(params: RequestParams) -> Void {
        EyNetManager.shareManager().download(params.url,to: DownloadRequest.suggestedDownloadDestination(for: .cachesDirectory)).downloadProgress { progress in
                    let map = ["progress" : progress.fractionCompleted, "requestId" : params.requestId] as [String : Any];
                    ChannelPlugin.invokeMethod(action: "f4f4cce92307e7f5", params: map);
                  }.validate().responseData { response in
                      switch response.result {
                      case .success(_):
                          var _response:Dictionary<String,String> = [:];
                          _response["path"] = response.destinationURL?.path;
                          let jsonData = JsonUtils.getJSONStringFromDictionary(dataObject: _response);
                          self.responseHanndler(callKey: params.callKey, type: "success", data: jsonData);
                          self.notifyComplete(requestId: params.requestId);
                          break;
                      case .failure(let error):
                          self.responseHanndler(callKey: params.callKey, type: "error", data: "");
                          self.notifyComplete(requestId: params.requestId);
                          print(error);
                          break;
                      }
                      
            };
    }
    
    func notifyComplete(requestId: String) -> Void {
        ChannelPlugin.invokeMethod(action: "812454d85242c263", params: requestId);
    }
    
    func responseHanndler(callKey: String, type: String, data: String) -> Void {
        var _response:Dictionary<String,String> = [:];
        _response["type"] = type;
        _response["data"] = data;
        let callData = JsonUtils.getJSONStringFromDictionary(dataObject: _response);
        ChannelPlugin.resultCall(resultKey: callKey, callbackData: callData);
    }
}
