import 'package:eyflutter_core/dio/beans/application_info.dart';
import 'package:eyflutter_core/event/on_eyflutter_config.dart';
import 'package:eyflutter_core/event/on_scheme_listener.dart';
import 'package:eyflutter_core/kit/parts/config_manager.dart';
import 'package:eyflutter_core/kit/utils/json_utils.dart';
import 'package:eyflutter_core/kit/utils/string/json_extension.dart';
import 'package:eyflutter_core/mq/cloud_channel_manager.dart';
import 'package:eyflutter_core/storage/memory_utils.dart';

mixin OnAddressConfig {
  /// api基础地址
  /// [hostType] 主机类型(根据hostType配置基础地址)
  String baseUrl({required int hostType});

  /// 资源基础url
  /// [resType] 资源类型(根据resType配置资源地址)
  String resBaseUrl({required int resType});

  /// 获取sdk配置app key、scret接口地址
  Future<String> getSdkKeyApiUrl();

  /// 设备信息url
  Future<String> deviceUrl();
}

class EyFlutter {
  factory EyFlutter() => _getInstance();

  static EyFlutter get instance => _getInstance();
  static EyFlutter? _instance;

  static EyFlutter _getInstance() {
    return _instance ??= EyFlutter._internal();
  }

  EyFlutter._internal();

  String _eyConfigKey = "f0b4595909fb23aa";
  String _appInfoKey = "e8f42f0a6a9de326";
  String _startTimeKey = "fba428faac04b4cd";
  String _schemeConfigKey = "02b4309b34ab4316";
  String _addressConfigKey = "0414193a4016a176";

  ApplicationInfo _appInfo = ApplicationInfo();

  void setAppInfo(ApplicationInfo? appInfo) {
    if (appInfo != null) {
      _appInfo = appInfo;
    }
  }

  ApplicationInfo get appInfo => _appInfo;

  /// 注册应用配置
  /// OnEyFlutterConfig实现类
  void regAppConfig<T extends OnEyFlutterConfig>(T? config) {
    if (config == null) {
      return;
    }
    ConfigManager.instance.addConfig(_eyConfigKey, config);
  }

  OnEyFlutterConfig? _getEyFlutterConfig() {
    var config = ConfigManager.instance.getConfig(_eyConfigKey);
    if (config is! OnEyFlutterConfig) {
      return null;
    }
    return config;
  }

  /// 获取eyflutter配置项
  OnEyFlutterConfig? get appConfig => _getEyFlutterConfig();

  /// 获取应用信息
  Future<ApplicationInfo> getAppInfo() async {
    var appJsonData = await CloudChannelManager.instance.send<String>(_appInfoKey);
    if (appJsonData.isEmptyJson) {
      return Future.value(ApplicationInfo());
    }
    var map = JsonUtils.fromJson(appJsonData!);
    var applicationInfo = ApplicationInfo.fromJson(map);
    _appInfo = applicationInfo;
    return Future.value(applicationInfo);
  }

  /// 设置应用启动时间
  void setStartTime({required int time}) {
    MemoryUtils.instance.set(_startTimeKey, time);
  }

  /// 获取应用启动时间
  int get startTime => MemoryUtils.instance.getInt(_startTimeKey);

  /// 注册scheme对象
  void registerScheme<T extends OnSchemeListener>(T config) {
    ConfigManager.instance.addConfig(_schemeConfigKey, config);
  }

  OnSchemeListener? _getSchemeConfig() {
    var config = ConfigManager.instance.getConfig(_schemeConfigKey);
    if (config is! OnSchemeListener) {
      return null;
    }
    return config;
  }

  /// 获取scheme配置
  OnSchemeListener? get schemeConfig => _getSchemeConfig();

  /// 注册地址配置
  /// OnAddressConfig实现类
  void regAddressConfig<T extends OnAddressConfig>(T? config) {
    if (config == null) {
      return;
    }
    ConfigManager.instance.addConfig(_addressConfigKey, config);
  }

  OnAddressConfig? _getAddressConfig() {
    var config = ConfigManager.instance.getConfig(_addressConfigKey);
    if (config is! OnAddressConfig) {
      return null;
    }
    return config;
  }

  /// 获取基础地址配置
  OnAddressConfig? get addressConfig => _getAddressConfig();
}
