import 'dart:isolate';

import 'package:eyflutter_core/eyflutter_core.dart';

typedef OnObserverCall = dynamic Function(Map<String, dynamic> params);
typedef OnFinishedCall = void Function(dynamic response);
typedef OnAccessCall = void Function(dynamic access, Map<String, dynamic> params);

class IsolateAccess {
  final dynamic access;
  final Map<String, dynamic> params;

  IsolateAccess(this.access, this.params);
}

class IsolateReceiver {
  /// 创建isolate线程并回调结果信息
  /// [observerCall]  订阅线程中函数
  /// [finishedCall] 结束后返回回调数据
  void execute({
    required OnObserverCall observerCall,
    OnAccessCall? accessCall,
    OnFinishedCall? finishedCall,
    Map<String, dynamic>? params,
  }) async {
    ReceivePort receivePort = ReceivePort();
    await Isolate.spawn(_observer, receivePort.sendPort);
    SendPort sendPort = await receivePort.first;
    var receive = await _sendReceive(sendPort, params ?? {}, observerCall, accessCall);
    if (finishedCall != null) {
      finishedCall(receive);
    }
    Logger.instance.info(receive);
  }

  void _observer(SendPort sendPort) async {
    ReceivePort receivePort = ReceivePort();
    sendPort.send(receivePort.sendPort);
    await for (var msg in receivePort) {
      if (msg is Map<String, dynamic>) {
        SendPort callbackPort = msg["port"];
        OnObserverCall observerCall = msg["observer"];
        Map<String, dynamic> params = msg["params"];
        var message = observerCall(params);
        SendPort callAccessPort = msg["accessPort"];
        var isolateAccess = IsolateAccess(message, params);
        callAccessPort.send(isolateAccess);
        callbackPort.send(message);
      }
    }
  }

  void sendAccess() {}

  Future _sendReceive(
      SendPort sendPort, Map<String, dynamic> params, OnObserverCall observerCall, OnAccessCall? accessCall) {
    ReceivePort accessReceive = ReceivePort();
    if (accessCall != null) {
      var first = accessReceive.first;
      if (first is IsolateAccess) {
        var access = first as IsolateAccess;
        accessCall(access.access, access.params);
      }
    }
    ReceivePort receivePort = ReceivePort();
    Map<String, dynamic> isolateParams = {};
    isolateParams["port"] = receivePort.sendPort;
    isolateParams["observer"] = observerCall;
    isolateParams["params"] = params;
    isolateParams["accessPort"] = accessReceive.sendPort;
    sendPort.send(isolateParams);
    return receivePort.first;
  }
}
