extension NumUtilsExtension on num? {
  /// 判断字符标识是否true(eg. true or 1)
  bool get isNumTrue {
    if (this == null) {
      return false;
    }
    if (this is num) {
      return this == 1;
    }
    return false;
  }

  /// 判断是否为数字类型
  bool get isNumNumeric {
    if (this == null) {
      return false;
    }
    if (this is num) {
      return true;
    }
    return false;
  }

  /// 转为double类型
  double get toNumDouble {
    if (this == null) {
      return 0;
    }
    try {
      if (this is double) {
        return this as double;
      } else {
        return double.tryParse("$this") ?? 0;
      }
    } catch (e) {
      return 0;
    }
  }

  /// 转为int类型
  int get toNumInt {
    if (this == null) {
      return 0;
    }
    try {
      if (this is num) {
        return (this ?? 0) ~/ 1;
      } else {
        return (num.tryParse("$this") ?? 0) ~/ 1;
      }
    } catch (e) {
      return 0;
    }
  }

  /// 是否为空或0
  bool get isNullOrZero => this == null || this == 0;

  /// 减数
  num reduce(num reduction) {
    if (this == null) {
      return -reduction;
    }
    return this ?? 0 - reduction;
  }

  /// 加数
  num add(num number) {
    if (this == null) {
      return number;
    }
    return this ?? 0 + number;
  }

  /// 取n位小数，不足补位
  /// [position] 要保留的位数
  /// [isCrop] true 直接裁剪 false 四舍五入
  String format({required int position, bool isCrop = false}) {
    String t = this.toString();
    if (position < 0) {
      return t;
    }
    if (t.contains(".")) {
      String t1 = t.split(".").last;
      if (t1.length >= position) {
        if (isCrop) {
          return t.substring(0, t.length - (t1.length - position));
        } else {
          return this?.toStringAsFixed(position) ?? "";
        }
      } else {
        String t2 = "";
        for (int i = 0; i < position - t1.length; i++) {
          t2 += "0";
        }
        return t + t2;
      }
    } else {
      String t3 = position > 0 ? "." : "";
      for (int i = 0; i < position; i++) {
        t3 += "0";
      }
      return t + t3;
    }
  }
}
