import 'dart:async';

import 'package:isolate/isolate.dart';

class ThreadPool {
  factory ThreadPool() => _getInstance();

  static ThreadPool get instance => _getInstance();
  static ThreadPool? _instance;

  static ThreadPool _getInstance() {
    return _instance ??= ThreadPool._internal();
  }

  ThreadPool._internal();

  Future<LoadBalancer> loadBalancer = LoadBalancer.create(8, IsolateRunner.spawn);

  /// 执行多线程微任务
  /// [doWork] 耗时任务
  /// [call] 耗时任务执行结果回调
  /// [argument] 任务执行传入参数
  void execute<R, P>(
      {required FutureOr<R> Function(P argument) doWork, Function(R response)? call, required P argument}) {
    loadBalancer.then((balancer) {
      balancer.run<R, P>(doWork, argument).then((response) {
        if (call != null) {
          call(response);
        }
      });
    });
  }

  /// 执行多线程同步返回微任务
  /// [doWork] 耗时任务
  /// [argument] 任务执行传入参数
  Future<R> executeOr<R, P>({required FutureOr<R> Function(P argument) doWork, required P argument}) async {
    var balancer = await loadBalancer;
    return balancer.run<R, P>(doWork, argument);
  }
}
